<?php
/*
Plugin Name: Birthdaze
Plugin URI: http://familypress.net/birthdaze/
Description: A very simple birthday plugin for WordPress, with a birthday manager, template function, and a widget.
Version: 0.5.1
Author: Isaac Wedin
Author URI: http://familypress.net/
*/

/* Copyright 2009, 2010 Isaac Wedin (email : isaac@familypress.net)
This program is free software; you can redistribute it and/or modify it under the terms of the GNU General Public License as published by the Free Software Foundation; either version 2 of the License, or (at your option) any later version. 

This program is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License for more details.

You should have received a copy of the GNU General Public License along with this program; if not, write to the Free Software Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/

// Tell the localization loader the name of the plugin so it can load a translation if one is needed and installed.
$plugin_dir = basename(dirname(__FILE__));
load_plugin_textdomain( 'birthdaze', 'wp-content/plugins/' . $plugin_dir, $plugin_dir );

function birthdaze_options_init() {
	register_setting('birthdazeoptions_options','birthdaze_options','birthdaze_sanitize');
}

function birthdaze_sanitize($input) {
	$input['capability'] = wp_filter_nohtml_kses($input['capability']);
	return $input;
}

// only those with the 'manage_options' capability will see the Options
function birthdaze_add_options_page() {
	add_options_page('Birthdaze', 'Birthdaze', 'manage_options', 'birthdaze_options', 'birthdaze_options_subpanel');
}

// generates the Birthdaze Options subpanel
function birthdaze_options_subpanel() {
	echo '
	<div class="wrap">
	<h2>' . __('Birthdaze Options','birthdaze') . '</h2>
	<form method="post" action="options.php">';
	settings_fields('birthdazeoptions_options');
	$birthdaze_options = get_option('birthdaze_options');
	if (empty($birthdaze_options['capability'])) {
		$birthdaze_options['capability'] = 'edit_posts';
	}
   if (!current_user_can($birthdaze_options['capability'])) { // they're an admin, so the capability *must* be wrong...
      echo "\n<div id='birthdaze-warning' class='updated fade'><p><strong>Error:</strong> The capability you have entered below is incorrect.</p></div>\n";
   }
	echo '
	<fieldset class="options">
	<table class="form-table">
		<tbody>
			<tr>
			<th scope="row">' . __('Capability required to use Birthdaze:','birthdaze') . '</th>
			<td><input type="text" size="20" name="birthdaze_options[capability]" value="' . $birthdaze_options['capability'] . '" /><br />
		' . __('You probably want to use <code>edit_posts</code>, but you could also use <code>upload_files</code>, <code>publish_posts</code> or any other capability you want.','birthdaze') . '</td>
		</tr>
	</table> 
	</fieldset>

	<p class="submit"><input type="submit" name="Submit" value="' . __('Update Options &raquo;','birthdaze') . '" /></p>
</form> 
</div>
';
}

// makes the Birthdaze Management page
function birthdaze_management() {
	$birthdaze_options = get_option('birthdaze_options');
	global $wpdb;
	$table_name = $wpdb->prefix . "birthdaze";
	if (isset($_POST['birthdaze_update'])) {
		$dels = 0;
		$updates = 0;
		foreach ((array)$_POST as $key=>$val) {
			if (is_array($val)) {
				if (isset($val['delete'])) {
					if (($val['delete'] == '1')) {
						if (birthdaze_delete($val['id']))
							$dels++;
					}
				} elseif (isset($val['fname']) && isset($val['bmonth']) && isset($val['bday'])) {
					if (!empty($val['fname']) && !empty($val['bmonth']) && !empty($val['bday'])) {
						$update_array = array();
						if (isset($val['id']))
							$update_array['id'] = $val['id'];
						$update_array['fname'] = $val['fname'];
						$update_array['lname'] = $val['lname'];
						$update_array['bmonth'] = $val['bmonth'];
						$update_array['bday'] = $val['bday'];
						if (birthdaze_update($update_array))
							$updates++;
					}
				}
			}
		}
		echo "<div class='updated'>";
		if ($dels > 0) {
			echo $dels . __(' records deleted. ','birthdaze');
		}
		if ($updates > 0) {
			echo $updates . __(' records updated.','birthdaze');
		}
		echo "</div>\n";
	}
	echo '
	<div class="wrap">
	<h2>' . __('Birthday Manager','birthdaze') . '</h2>
	<p>' . __('Add a birthday or edit the existing birthdays. Enter at least a first name and month.','birthdaze') . '</p>
	<form name="birthdaze" method="post">
	<p class="submit"><input type="submit" name="Submit" value="' . __('Update','birthdaze') . ' &raquo;" /></p>
	<input type="hidden" name="birthdaze_update" value="update" />
	<table>
	<tr>
		<th scope="col">' . __('First','birthdaze') . '</th>
		<th scope="col">' . __('Last','birthdaze') . '</th>
		<th scope="col">' . __('Month','birthdaze') . '</th>
		<th scope="col">' . __('Day','birthdaze') . '</th>
		<th scope="col">' . __('Delete','birthdaze') . '</th>
	</tr>
	<tr>
		<td><input type="text" name="row1[fname]" value="" size="15" /></td>
		<td><input type="text" name="row1[lname]" value="" size="15" /></td>
		<td><input type="text" name="row1[bmonth]" value="" size="2" /></td>
		<td><input type="text" name="row1[bday]" value="" size="2" /></td>
		<td></td>
	</tr>
	';
	if ($contacts = $wpdb->get_results("SELECT * FROM $table_name ORDER BY lname,fname",ARRAY_A)) {
		$i = 2;
		foreach ((array)$contacts as $contact) {
			echo '<tr>
		<td><input type="hidden" name="row' . $i . '[id]" value="' . $contact['id'] . '" /><input type="text" name="row' . $i . '[fname]" value="' . $contact['fname'] . '" size="15" /></td>
		<td><input type="text" name="row' . $i . '[lname]" value="' . $contact['lname'] . '" size="15" /></td>
		<td><input type="text" name="row' . $i . '[bmonth]" value="' . $contact['bmonth'] . '" size="2" /></td>
		<td><input type="text" name="row' . $i . '[bday]" value="' . $contact['bday'] . '" size="2" /></td>
		<td><input type="checkbox" name="row' . $i . '[delete]" value="1" /></td>
	</tr>';
			$i++;
		}
	}
	echo '
		</table>
		<p class="submit"><input type="submit" name="Submit" value="' . __('Update','birthdaze') . ' &raquo;" /></p>
		</form>
	</div>
	';
}

function birthdaze_add_management_page() {
	$birthdaze_options = get_option('birthdaze_options');
	add_management_page('Birthdays', 'Birthdays', $birthdaze_options['capability'], basename(__FILE__), 'birthdaze_management');
}

// install a table in the db for birthdaze data
function birthdaze_table_install() {
	global $table_prefix, $wpdb;
	$table_name = $table_prefix . "birthdaze";
	$sql = "CREATE TABLE ".$table_name." (
		id mediumint(9) NOT NULL AUTO_INCREMENT,
		fname varchar(55) NULL,
		lname varchar(55) NULL,
		bmonth tinyint(2) NULL,
		bday tinyint(2) NULL,
		UNIQUE KEY id (id)
	);";
	require_once(ABSPATH . 'wp-admin/includes/upgrade.php');
	dbDelta($sql);
}

// run the table installer when the plugin is activated
if (isset($_GET['activate']) && $_GET['activate'] == 'true') {
	add_action('init', 'birthdaze_table_install');
}

function birthdaze_update($update_array) {
	global $wpdb, $birthdaze_options, $table_prefix;
	$table_name = $table_prefix . "birthdaze";
	foreach ($update_array as $key=>$val) {
		$update_array[$key] = $wpdb->escape($val);
	}
	if (isset($update_array['id'])) {
		if ($wpdb->get_var("SELECT id FROM $table_name WHERE binary id = '" . $update_array['id'] . "'")) {
			return $wpdb->query("UPDATE $table_name SET 
				fname = '" . $update_array['fname'] . "', 
				lname = '" . $update_array['lname'] . "', 
				bmonth = '" . $update_array['bmonth'] . "',
				bday = '" . $update_array['bday'] . "'
				WHERE binary id = '" . $update_array['id'] . "'");
		}
	} else {
		return $wpdb->query("INSERT INTO $table_name SET 
	fname = '" . $update_array['fname'] . "',
	lname = '" . $update_array['lname'] . "',
	bmonth = '" . $update_array['bmonth'] . "',
	bday = '" . $update_array['bday'] . "'");
	}
}

function birthdaze_delete($id) {
	global $wpdb, $birthdaze_options, $table_prefix;
	$table_name = $table_prefix . "birthdaze";
	$id = $wpdb->escape($id);
	if ($wpdb->query("DELETE FROM $table_name WHERE binary id = '$id'")) {
		return TRUE;
	} else {
		return FALSE;
	}
}

//wrappers to keep old functions working
function birthdaze_thismonth($before='<li>',$after='</li>',$none='None found.') {
	birthdaze_print($before,$after,$none,TRUE,'month','F j');
}

function birthdaze_allofem($before='<li>',$after='</li>',$none='None found.') {
	birthdaze_print($before,$after,$none,TRUE,'all','F j');
}

//birthday printing wrapper function
function birthdaze_print($before='<li>',$after='</li>',$none='None found.',$initial=TRUE,$time='all',$bdformat='F j') {
	$bdayslist = birthdaze_return($before,$after,$none,$initial,$time,$bdformat);
	echo $bdaylist;
}

//do-it-all birthday return function
function birthdaze_return($before='<li>',$after='</li>',$none='None found.',$initial=TRUE,$time='month',$bdformat='F j') {
	global $wpdb;
	if ($initial == 'FALSE')
		$initial = FALSE;
	$table_name = $wpdb->prefix . "birthdaze";
	$today = getdate();
	if ($time == 'month') {
		$bdays = $wpdb->get_results("SELECT fname,lname,bmonth,bday FROM $table_name WHERE bmonth = MONTH(CURDATE()) ORDER BY bday",ARRAY_A);
	} else {
		$bdays = $wpdb->get_results("SELECT fname,lname,bmonth,bday FROM $table_name ORDER BY bmonth,bday",ARRAY_A);
	}
	if ($bdays == FALSE) {
		return $none;
	} else {
		$bdayslist = '';
		foreach ((array)$bdays as $bday) {
			if ($bday['bday'] < 10) {
				$day = '0' . $bday['bday'];
			} else {
				$day = $bday['bday'];
			}
			if ($bday['bmonth'] < 10) {
				$month = '0' . $bday['bmonth'];
			} else {
				$month = $bday['bmonth'];
			}
			$bdaytime = strtotime($today['year'] . $month . $day);
			$bday_text = date_i18n($bdformat,$bdaytime);
			if (!empty($bday['lname'])) {
				if ($initial) {
					$lname = " " . substr($bday['lname'],0,1);
				} else {
					$lname = ' ' . $bday['lname'];
				}
			} else {
				$lname = '';
			}
			$curtime = time();
			$curdate = date('Ymd',$curtime);
			$curtime = strtotime($curdate);
			if ($curtime == $bdaytime) {
				$bdaytoday = 'bdaytoday';
			} else {
				$bdaytoday = 'nobdaytoday';
			}
			$bdayoutput = $before . '<span class="' . $bdaytoday . '">' . $bday['fname'] . $lname . ": " . $bday_text . '</span>' . $after . "\n";
			if ($time == 'week') {
				$weekfromnow = $curtime + (7 * 24 * 60 * 60);
				if (($bdaytime >= $curtime) && ($bdaytime < $weekfromnow))
					$bdayslist .= $bdayoutput;
			} else {
				$bdayslist .= $bdayoutput;
			}
		}
		return $bdayslist;
	}
}

// birthdazeWidget Class 
class birthdazeWidget extends WP_Widget {
   // constructor
   function birthdazeWidget() {
      $widget_ops = array('description' => __( "Displays upcoming birthdays", "birthdaze" ) );
      $this->WP_Widget('birthdaze', __('Birthdays', 'birthdaze'), $widget_ops);
   }
   // @see WP_Widget::widget  - display the widget
	function widget($args, $instance) {
		extract($args);
		$title = apply_filters('widget_title', $instance['title']);
      $linitial = $instance['linitial'];
      $bdformat = $instance['bdformat'];
      $time = $instance['time'];
		$after = stripslashes(html_entity_decode($instance['after'],ENT_QUOTES));
		$afterlist = stripslashes(html_entity_decode($instance['afterlist'],ENT_QUOTES));
		$before = stripslashes(html_entity_decode($instance['before'],ENT_QUOTES));
		$beforelist = stripslashes(html_entity_decode($instance['beforelist'],ENT_QUOTES));
      ?>
		<?php echo $before_widget; ?>
           <?php if ( $title )
					echo $before_title . $title . $after_title; ?>
				<?php if ( function_exists('birthdaze_return') ) {
					if ($birthday_list = birthdaze_return($before,$after,'None found',$linitial,$time,$bdformat))
						echo $beforelist . $birthday_list . $afterlist;
            } ?>
      <?php echo $after_widget; ?>
      <?php
   } // function widget

	// @see WP_Widget::update  - process the options
   function update($new_instance, $old_instance) {
      $instance = $old_instance;
		$instance['title'] = strip_tags($new_instance['title']);
		$instance['linitial'] = strip_tags($new_instance['linitial']);
		$instance['bdformat'] = strip_tags($new_instance['bdformat']);
		$instance['time'] = strip_tags($new_instance['time']);
		if ( current_user_can('unfiltered_html') ) {
			$instance['before'] =  $new_instance['before'];
			$instance['beforelist'] =  $new_instance['beforelist'];
			$instance['after'] =  $new_instance['after'];
			$instance['afterlist'] =  $new_instance['afterlist'];
		} else {
         $instance['before'] = stripslashes( wp_filter_post_kses( addslashes($new_instance['before']) ) );
         $instance['beforelist'] = stripslashes( wp_filter_post_kses( addslashes($new_instance['beforelist']) ) );
         $instance['after'] = stripslashes( wp_filter_post_kses( addslashes($new_instance['after']) ) );
         $instance['afterlist'] = stripslashes( wp_filter_post_kses( addslashes($new_instance['afterlist']) ) );
      }
      return $instance;
   } // function update

   // @see WP_Widget::form  - output the options form
   function form($instance) {
		$instance = wp_parse_args( (array) $instance,  array('title'=>'', 'linitial'=>'TRUE', 'time'=>'month', 'bdformat'=>'F j', 'widgets', 'before'=>'<li>', 'beforelist'=>'<ul>', 'afterlist'=>'</ul>', 'after'=>'</li>') );
      $title = htmlspecialchars($instance['title'], ENT_QUOTES);
      $linitial = $instance['linitial'];
      $time = $instance['time'];
      $bdformat = $instance['bdformat'];
      $before = format_to_edit($instance['before']);
      $beforelist = format_to_edit($instance['beforelist']);
      $after = format_to_edit($instance['after']);
      $afterlist = format_to_edit($instance['afterlist']);
      ?>
         <p><label for="<?php echo $this->get_field_id('title'); ?>"><?php _e('Title:','birthdaze'); ?> <input class="widefat" id="<?php echo $this->get_field_id('title'); ?>" name="<?php echo $this->get_field_name('title'); ?>" type="text" value="<?php echo $title; ?>" /></label></p>

         <p><label for="<?php echo $this->get_field_id('before'); ?>"><?php _e('Before each birthday:','birthdaze'); ?> <input class="widefat" id="<?php echo $this->get_field_id('before'); ?>" name="<?php echo $this->get_field_name('before'); ?>" type="text" value="<?php echo $before; ?>" /></label></p>

         <p><label for="<?php echo $this->get_field_id('after'); ?>"><?php _e('After each birthday:','birthdaze'); ?> <input class="widefat" id="<?php echo $this->get_field_id('after'); ?>" name="<?php echo $this->get_field_name('after'); ?>" type="text" value="<?php echo $after; ?>" /></label></p>

         <p><label for="<?php echo $this->get_field_id('beforelist'); ?>"><?php _e('Before the list:','birthdaze'); ?> <input class="widefat" id="<?php echo $this->get_field_id('beforelist'); ?>" name="<?php echo $this->get_field_name('beforelist'); ?>" type="text" value="<?php echo $beforelist; ?>" /></label></p>

         <p><label for="<?php echo $this->get_field_id('afterlist'); ?>"><?php _e('After the list:','birthdaze'); ?> <input class="widefat" id="<?php echo $this->get_field_id('afterlist'); ?>" name="<?php echo $this->get_field_name('afterlist'); ?>" type="text" value="<?php echo $afterlist; ?>" /></label></p>

         <p><label for="<?php echo $this->get_field_id('bdformat'); ?>"><?php _e('Birthday format:','birthdaze'); ?> <input class="widefat" id="<?php echo $this->get_field_id('bdformat'); ?>" name="<?php echo $this->get_field_name('bdformat'); ?>" type="text" value="<?php echo $bdformat; ?>" /></label></p>

			 <p><label for="<?php echo $this->get_field_id('linitial'); ?>"><?php _e('Use initial for last name:','birthdaze'); ?><select id="<?php echo $this->get_field_id('linitial'); ?>" name="<?php echo $this->get_field_name('linitial'); ?>">
				<option value="TRUE"<?php if ($linitial == 'TRUE') echo ' selected="selected"'; ?>><?php _e('yes','birthdaze'); ?></option>
				<option value="FALSE"<?php if ($linitial == 'FALSE') echo ' selected="selected"'; ?>><?php _e('no','birthdaze'); ?></option>
			 </select></label></p>

			 <p><label for="<?php echo $this->get_field_id('time'); ?>"><?php _e('Time period:','birthdaze'); ?><select id="<?php echo $this->get_field_id('time'); ?>" name="<?php echo $this->get_field_name('time'); ?>">
				<option value="month"<?php if ($time == 'month') echo ' selected="selected"'; ?>><?php _e('this month','birthdaze'); ?></option>
				<option value="week"<?php if ($time == 'week') echo ' selected="selected"'; ?>><?php _e('this week','birthdaze'); ?></option>
				<option value="all"<?php if ($time == 'all') echo ' selected="selected"'; ?>><?php _e('all','birthdaze'); ?></option>
			 </select></label></p>

      <?php
   } // function form
} // class birthdazeWidget

// register birthdazecommWidget widget
add_action('widgets_init', create_function('', 'return register_widget("birthdazeWidget");'));

function birthdaze_shortcode_proc($atts) {
	extract(shortcode_atts(array(
		'before' => '<li>',
		'after' => '</li>',
		'time' => 'all',
		'bdformat' => 'F j',
		'initial' => 'TRUE',
		'notfound' => 'None found.',
	), $atts));
	$bdformat = stripslashes(html_entity_decode($bdformat,ENT_QUOTES));
	$before = stripslashes(html_entity_decode($before,ENT_QUOTES));
	$after = stripslashes(html_entity_decode($after,ENT_QUOTES));
	$birthday_list = birthdaze_return($before,$after,$notfound,$initial,$time,$bdformat);
	return $birthday_list;
}

add_action('admin_menu', 'birthdaze_add_options_page');
add_action('admin_menu', 'birthdaze_add_management_page');
add_action('admin_init','birthdaze_options_init');
add_shortcode('birthdaze','birthdaze_shortcode_proc');

?>
